{
==============================================================================
 FlexMenu - Release 99/1  13.02.1999
 for Delphi 4, should also run with Delphi 3

------------------------------------------------------------------------------

 FlexMenu is FREEWARE. Freeware means, that you can use this software without
 paying anything for it, but it is not public domain! This software is
 protected through the law of the Federal Republic of Germany, the European
 Union and other countries.

 (C)1999 by Peter Hellinger Software, All rights are reserved.

 Peter Hellinger Software, Zerzabelshofstrasse 41, D-90480 Nrnberg

 email: mail@helli.de
 homepage: http://www.helli.de

==============================================================================

 Installation:
 Copy FLEXMENU.PAS AND FLEXMENU.DCR into your library path.
 Select Components -> Install Components from the Delphi main menu and put
 FLEXMENU.PAS to a package of your choice.

------------------------------------------------------------------------------

 TFlexMenu enables you to add or remove itmes on a menu dynamicly. Useful for
 flexibe menu entries like MRU or window lists. Features automatic numbering
 of the added menuitems and automatic add/remove of separator lines. TFlexMenu
 is a non visual component.

 TFlexMenu provides the following properties and methods:

 Properties:
 ===========

 MaxItems	Number of menuitems that can be added to the menu. -1 means
                that there is no limit. If you set a Limit, not more than
                MaxItems entries are added to the parent menu.


 ParentMenu     The TMenuItem that the Subitems are added to,


 NumberItems    TRUE means, that the entries are numberd by TFlexMenu. Example:

                   1 Entry No. one
                   2 Entry No. two
                   3 Entry No. three
                   4 Entry No. four

                If you remove "Entry No. two" the items will be renumberd and
                look like this:

                   1 Entry No. one
                   2 Entry No. three
                   3 Entry No. four

                If NumberItems is TRUE, the Number is the Shotcut to the
                MenuItem!


 CreateLine     IF true, The first added menuitem also adds a separator line
                to the menu. The line is automaticly removed, if all menus
                that are added are removed.

 Methodes:
 =========

 Create         Creates the component

 Destroy        Destroys the component

 Add            Adds a menuitem to the parent menu. The methode returns the
                added Menuitem as a TMenuItem object. Parameters:

                title =    The caption of the menuitem
                checked =  The item is checked
                enabled =  The item is enabled
                tag =      The property Tag of the TMenuItem
                click =    The event handler for the OnClick event of the
                           TMenuItem

 Remove         Removes the menuitem from the parent menu.
                Parameters:

                item =     The TMenuItem, returned by Add or GetItem.

 ChangeCaption  Changes the Caption OF the menuitem. You should always
                use this methode instead manipulating the menuitem in direct
                way, because ChangeCaption updates the numbering Top.
                Parameters:

                item =     The TMenuItem, returned by Add or GetItem
                title =    The new caption of the menuitem

 GetItem        Returns the TMenuItem with the specific number from the list
                of the added menuitems, or NIL if the item not exist.
                Parameters:

                item =     The Number of the wanted TMenuItem

==============================================================================
}

UNIT FlexMenu;

INTERFACE

USES
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Menus;

TYPE Str80 = STRING[80];

TYPE PtrMenuEntry = ^tMenuEntry;
     tMenuEntry =   RECORD
                     Caption: Str80;
                     item: TMenuItem;
                    END;


TYPE
  TFlexMenu = class(TComponent)
  PRIVATE
   { Private-Deklarationen }
   Flist: TList;
   FMenuLine: TMenuItem;
   FAdded: BOOLEAN;
  PROTECTED
   { Protected-Deklarationen }
   FMenu: TMenuItem;
   FMax: INTEGER;
   FNumber: BOOLEAN;
   FLine: BOOLEAN;
   PROCEDURE NumberEntries;
  PUBLIC
   { Public-Deklarationen }
  PUBLISHED
   { Published-Deklarationen }
   CONSTRUCTOR Create (AOwner: TComponent); OVERRIDE;
   DESTRUCTOR Destroy; OVERRIDE;
   PROPERTY MaxItems: INTEGER read FMax write FMax;
   PROPERTY ParentMenu:  TMenuItem read FMenu write FMenu;
   PROPERTY NumberItems: BOOLEAN read FNumber write FNumber;
   PROPERTY CreateLine: BOOLEAN read FLine write FLine;
   FUNCTION Add (CONST title: STRING; Checked, Enabled: BOOLEAN; tag: INTEGER; click: TNotifyEvent): TMenuItem;
   PROCEDURE ChangeCaption (item: TMenuItem; title: STRING);
   PROCEDURE Remove (item: TMenuItem);
   FUNCTION GetItem (item: INTEGER): TMenuItem;
  END;

PROCEDURE Register;

IMPLEMENTATION

PROCEDURE Register;
BEGIN
 RegisterComponents('Samples', [TFlexMenu]);
END;

CONSTRUCTOR TFlexMenu.Create (AOwner: TComponent);
BEGIN
 INHERITED;
 FMax:= -1;
 FNumber:= FALSE;
 FLine:= FALSE;
 FMenuLine:= NIL;
 FMenu:= NIL;
 FList:= TList.Create;
 FAdded:= FALSE;
END;

DESTRUCTOR TFlexMenu.Destroy;
VAR i: INTEGER;
BEGIN
 FOR i:= 0 TO FList.Count - 1 DO  Dispose (FList[i]);
 FList.Free;
 INHERITED;
END;

FUNCTION TFlexMenu.GetItem (item: INTEGER): TMenuItem;
VAR m: PtrMenuEntry;
BEGIN
 m:= FList.Items[item];
 IF m <> NIL THEN Result:= m.item
             ELSE Result:= NIL;
END;

PROCEDURE TFlexMenu.NumberEntries;
VAR m: PtrMenuEntry;
    s: STRING;
    i: INTEGER;
BEGIN
 // Captions anpassen, wenn Nummerierung gewnscht
 IF FNumber THEN BEGIN
  FOR i:= 0 TO FList.Count - 1 DO BEGIN
   m:= FList.Items[i];
   IF m <> NIL THEN BEGIN
    s:= '&' + IntToStr (i+1) + ' ' + m^.Caption;
    m.item.Caption:= s;
   END;
  END;
 END;
END;

FUNCTION TFlexMenu.Add (CONST title: STRING;
                        Checked, Enabled: BOOLEAN;
                        tag: INTEGER;
                        click: TNotifyEvent): TMenuItem;
VAR p: PtrMenuEntry;
BEGIN
 Result:= NIL;
 IF (FMax > -1) AND (FList.Count >= FMax) THEN EXIT;
 IF (NOT FAdded) AND FLine THEN BEGIN
  FMenuLine:= NewLine;
  FMenu.Add (FMenuline);
  FAdded:= TRUE;
 END;
 New (p);
 IF p <> NIL THEN BEGIN
  p^.Caption:= title;
  p^.item:= NewItem (title, 0, checked, enabled, click, 0, '');
  p^.item.Tag:= tag;
  Result:= p^.item;
  FList.Add (p); // Eintrag merken
  FMenu.Add (p^.item);
  NumberEntries;
 END;
END;

PROCEDURE TFlexMenu.Remove (item: TMenuItem);
VAR p: PtrMenuEntry;
    i: INTEGER;
BEGIN
 FOR i:= 0 TO FList.Count - 1 DO BEGIN
  p:= FList.Items[i];
  IF p <> NIL THEN BEGIN
   IF p^.item = item THEN BEGIN
    FMenu.Remove (p^.item);
    Dispose (p);
    FList.Delete (i);
    FList.Pack;
    NumberEntries;
    IF (FList.Count <= 0) AND Assigned (FMenuLine) THEN BEGIN
     FMenu.Remove (FMenuLine);
     FMenuLine:= NIL;
     FAdded:= FALSE;
    END;
    EXIT;
   END;
  END;
 END;
END;

PROCEDURE TFlexMenu.ChangeCaption (item: TMenuItem; title: STRING);
VAR p: PtrMenuEntry;
    i: INTEGER;
BEGIN
 FOR i:= 0 TO FList.Count - 1 DO BEGIN
  p:= FList.Items[i];
  IF p <> NIL THEN BEGIN
   IF p^.item = item THEN BEGIN
    p^.Caption:= title;
    p^.item.Caption:= p^.Caption;
    NumberEntries;
    EXIT;
   END;
  END;
 END;
END;

END.
